/* Who Watches the Watchmen?
 *
 * Special case: n = 1 has no solution.
 *
 * It's always better to rotate all sentries than move a single one.
 *
 * If we could ignore occlusions, this would be a textbook maximum matching
 * problem. Create a bipartite graph with N nodes on each side. A sentry can
 * view another sentry with a cost of 0 if it is already looking at it, and
 * cost 1 if it could be rotated otherwise.
 *
 * As it turns out, the occlusions usually don't matter. We can prove if there
 * are an even number of sentries or there exist three sentries which are not
 * collinear, we can always find a solution using only rotations.
 *
 * The only special case is then the case where we have an odd number of
 * sentries, all of which are collinear. In this case, we try moving each
 * sentry out of the line. The remaining sentries can either buddy up into
 * loops of size 2 or be part of a big loop with the moved sentry. Use prefix
 * sums to speed up counting how many sentries on either end of the big loop
 * need to rotate, and how many sentries on the loop need to rotate. This part
 * also requires ray-ray intersection to tell if we can move a sentry to a
 * point where it sees one end of the loop and the other end sees it.
 */
#include <algorithm>
#include <array>
#include <cassert>
#include <cstdint>
#include <iostream>
#include <limits>
#include <vector>

template <typename T, size_t N>
struct Point {
    std::array<T, N> c;

    Point& operator+=(const Point& other) {
        for (auto i = size_t{}; i < N; ++i) {
            c[i] += other.c[i];
        }

        return *this;
    }

    Point& operator-=(const Point& other) {
        for (auto i = size_t{}; i < N; ++i) {
            c[i] -= other.c[i];
        }

        return *this;
    }

    Point& operator*=(T x) {
        for (auto& v : c) {
            v *= x;
        }

        return *this;
    }

    T dist2() const {
        T res{};
        for (auto v : c) {
            res += v * v;
        }

        return res;
    }

    auto operator<=>(const Point& other) const = default;

    explicit operator bool() const {
        for (auto x : c) {
            if (x) {
                return true;
            }
        }

        return false;
    }

    friend std::istream& operator>>(std::istream& is, Point& p) {
        for (auto& x : p.c) {
            is >> x;
        }

        return is;
    }

    friend std::ostream& operator<<(std::ostream& os, const Point& p) {
        for (const auto& x : p.c) {
            os << x << ' ';
        }

        return os;
    }
};

template <typename T, size_t N>
Point<T, N> operator-(const Point<T, N>& p) {
    auto q{p};
    for (auto& x : q.c)
        x = -x;
    return q;
}

template <typename T, size_t N>
Point<T, N> operator+(const Point<T, N>& lhs, const Point<T, N>& rhs) {
    return Point<T, N>{lhs} += rhs;
}

template <typename T, size_t N>
Point<T, N> operator-(const Point<T, N>& lhs, const Point<T, N>& rhs) {
    return Point<T, N>{lhs} -= rhs;
}

template <typename T, size_t N>
Point<T, N> operator*(const Point<T, N>& lhs, T rhs) {
    return Point<T, N>{lhs} *= rhs;
}

template <typename T, size_t N>
Point<T, N> operator*(T lhs, const Point<T, N>& rhs) {
    return Point<T, N>{rhs} *= lhs;
}

template <typename T, size_t N>
T dot(const Point<T, N>& lhs, const Point<T, N>& rhs) {
    T res{};
    for (auto i = size_t{}; i < N; ++i) {
        res += lhs.c[i] * rhs.c[i];
    }

    return res;
}

template <typename T>
Point<T, 3> cross(const Point<T, 3>& lhs, const Point<T, 3>& rhs) {
    return {
        lhs.c[1] * rhs.c[2] - lhs.c[2] * rhs.c[1],
        lhs.c[2] * rhs.c[0] - lhs.c[0] * rhs.c[2],
        lhs.c[0] * rhs.c[1] - lhs.c[1] * rhs.c[0]
    };
}

using P3 = Point<int64_t, 3>;
using Sentry = std::array<P3, 2>;

bool sameDir(const P3& a, const P3& b) {
    assert(bool{a});
    assert(bool{b});

    return !bool{cross(a, b)} && dot(a, b) > 0;
}

// does p + v hit t?
bool faces(const P3& p, const P3 & v, const P3& t) {
    return sameDir(v, t - p);
}

// returns true if a + tv intersects b + sw for some t, s >= 0.
// assumes that a != b
bool doRaysIntersect(const P3& a, const P3& v, const P3& b, const P3& w) {
    P3 d = b - a;
    assert(bool{d});
    assert(bool{v});
    assert(bool{w});

    // vx t - wx t = bx - ax
    // vy t - wy t = by - ay
    int64_t tNum = d.c[0] * -w.c[1] - -w.c[0] * d.c[1];
    int64_t sNum = v.c[0] * d.c[1] - d.c[0] * v.c[1];
    int64_t den = v.c[0] * -w.c[1] - -w.c[0] * v.c[1];

    // lines are parallel or the same
    // TODO: check if it's ever possible for lines to be the same
    if (den == 0) {
        return false;
    }

    if (den < 0) {
        tNum = -tNum;
        sNum = -sNum;
        den = -den;
    }

    // lines intersect, but not rays
    if (tNum < 0 || sNum < 0) {
        return false;
    }

    return a * den + v * tNum == b * den + w * sNum;
}

bool allOnSameLine(const std::vector<Sentry>& sentries) {
    assert(sentries.size() > 1);
    P3 d = sentries[1][0] - sentries[0][0];
    for (auto i = 2; i < sentries.size(); ++i) {
        P3 f = sentries[i][0] - sentries[0][0];
        if (bool{cross(d, f)}) {
            return false;
        }
    }

    return true;
}

void solveLine(std::vector<Sentry> sentries) {
    int32_t n = sentries.size();
    assert(n > 1 && n % 2 == 1);

    std::array<P3, 2> dirs{sentries[1][0] - sentries[0][0], sentries[0][0] - sentries[1][0]};
    // First, sort the sentries along the line.
    std::sort(sentries.begin(), sentries.end(), [&](const Sentry& a, const Sentry& b) -> bool {
        return dot(dirs[0], a[0] - sentries[0][0]) < dot(dirs[0], b[0] - sentries[0][0]); 
    });

    constexpr int64_t INF = 2000;
    int64_t ans = INF;

    std::array<std::vector<int64_t>, 2> evenCost{};
    std::array<std::vector<int64_t>, 2> turnCost{};
    // Try each sentry in turn as the one to move
    for (auto movedIdx = 0; movedIdx < n; ++movedIdx) {
        // evenCost[0][i] is the min cost to fix the sentries up to but not including i
        evenCost[0].assign(n + 1, INF);
        evenCost[0][0] = 0;
        for (auto i = 0; i < n; ++i) {
            if (i == movedIdx) {
                assert(0 <= i && i < n);
                evenCost[0][i + 1] = evenCost[0][i];
                continue;
            }
            int32_t j = i - 1 == movedIdx ? i - 2 : i - 1;
            if (j < 0) {
                continue;
            }

            evenCost[0][i + 1] = !faces(sentries[i][0], sentries[i][1], sentries[j][0]) 
                               + !faces(sentries[j][0], sentries[j][1], sentries[i][0])
                               + evenCost[0][j];
        }

        // evenCost[1][i] is the min cost to fix the sentries from i to the end, including i
        evenCost[1].assign(n + 1, INF);
        evenCost[1][n] = 0;
        for (auto i = n - 1; i >= 0; --i) {
            if (i == movedIdx) {
                assert(0 <= i && i < n);
                evenCost[1][i] = evenCost[1][i + 1];
                continue;
            }

            int32_t j = i + 1 == movedIdx ? i + 2 : i + 1;
            if (j >= n) {
                continue;
            }

            evenCost[1][i] = !faces(sentries[i][0], sentries[i][1], sentries[j][0])
                           + !faces(sentries[j][0], sentries[j][1], sentries[i][0])
                           + evenCost[1][j + 1];
        }

        // turnCost[k] is a prefix sum array of how many sentries need to turn to face in dirs[k]
        for (auto k = 0; k < 2; ++k) {
            P3 dir = dirs[k];
            turnCost[k].assign(n + 1, 0);
            for (auto i = 0; i < n; ++i) {
                turnCost[k][i + 1] = turnCost[k][i];
                if (i != movedIdx && !sameDir(dir, sentries[i][1])) {
                    ++turnCost[k][i + 1];
                }
            }
        }

        // Now, fix the two endpoints of the odd loop, use evenCost to find the
        // total cost in O(1) amortized time.
        P3 movedV = sentries[movedIdx][1];
        for (auto lhs = 0; lhs < n; ++lhs) if (lhs != movedIdx) {
            for (auto rhs = lhs + 1; rhs < n; ++rhs) if (rhs != movedIdx) {
                for (auto turn = 0; turn < 2; ++turn) {
                    int64_t lastIdx = turn ? lhs : rhs;
                    int64_t turns = turnCost[turn][rhs + 1] - turnCost[turn][lhs] - (turnCost[turn][lastIdx + 1] - turnCost[turn][lastIdx]);

                    P3 dir = dirs[turn];
                    P3 startP = sentries[lhs ^ rhs ^ lastIdx][0];

                    // last thing to consider is the last sentry and the moved sentry.
                    // If both are viewing the line, both need to turn.
                    // If one is viewing the line, then only one needs to turn
                    // If neither are viewing the line and their rays intersect, then no turns
                    bool movedViewLine = !bool{cross(movedV, dir)};
                    bool lastViewLine = !bool{cross(sentries[lastIdx][1], dir)};
                    int32_t cur = 0;
                    if (movedViewLine && lastViewLine) {
                        cur = 2;
                    } else if (movedViewLine || lastViewLine || !doRaysIntersect(startP, -movedV, sentries[lastIdx][0], sentries[lastIdx][1])) {
                        cur = 1;
                    }

                    ans = std::min(ans, 1000 + evenCost[0][lhs] + evenCost[1][rhs + 1] + turns + cur);
                }
            }
        }
    }

    std::cout << ans << '\n';
}

// Adapted from KACTL
template <typename Cost>
std::pair<Cost, std::vector<int32_t>> matching(const std::vector<Cost> costs, size_t n, size_t m) {
    constexpr Cost INF = std::numeric_limits<Cost>::max();
    ++n; ++m;
    auto getCost = [&](size_t lhs, size_t rhs) -> Cost {
        return costs[(m - 1) * lhs + rhs];
    };

    if (costs.empty()) return {0, {}};
    std::vector<Cost> u(n, {}), v(m, {});
    using vi = std::vector<int32_t>;
    vi p(m), ans(n - 1);
    for (auto i = 1; i < n; ++i) {
        p[0] = i;
        int j0 = 0; // add "dummy" worker 0
        std::vector<Cost> dist(m, INF);
        vi pre(m, -1);
        std::vector<bool> done(m + 1);
        do { // dijkstra
            done[j0] = true;
            int i0 = p[j0], j1;
            Cost delta = INF;
            for(auto j = 1; j < m; ++j) if (!done[j]) {
                Cost cur = getCost(i0 - 1, j - 1) - u[i0] - v[j];
                if (cur < dist[j]) dist[j] = cur, pre[j] = j0;
                if (dist[j] < delta) delta = dist[j], j1 = j;
            }
            for (auto j = 0; j < m; ++j) {
                if (done[j]) u[p[j]] += delta, v[j] -= delta;
                else dist[j] -= delta;
            }
            j0 = j1;
        } while (p[j0]);
        while (j0) { // update alternating path
            int j1 = pre[j0];
            p[j0] = p[j1], j0 = j1;
        }
    }
    for (auto j = 1; j < m; ++j) {
        if (p[j]) ans[p[j] - 1] = j - 1;
    }
    return {-v[0], ans}; // min cost
}

void solveGeneral(std::vector<Sentry> sentries) {
    // build a cost matrix. Assign INF if a sentry cannot view another sentry.
    // Find the min cost matching. Return.
    auto n = sentries.size();
    constexpr int64_t INF = 1000;

    std::vector<int64_t> costs(n * n, INF);
    std::vector<P3> deltas(n);
    for (auto i = size_t{}; i < n; ++i) {
        for (auto j = size_t{}; j < n; ++j) {
            deltas[j] = sentries[j][0] - sentries[i][0];
        }

        // TODO: see if a map is faster
        for (auto j = size_t{}; j < n; ++j) {
            if (i == j) continue;

            bool valid = true;
            for (auto k = size_t{}; k < n; ++k) {
                if (k == i || k == j) continue;

                if (sameDir(deltas[j], deltas[k]) && deltas[k].dist2() < deltas[j].dist2()) {
                    valid = false;
                    break;
                }
            }

            if (valid) {
                costs[n * i + j] = sameDir(sentries[i][1], deltas[j]) ? 0 : 1;
            }
        }
    }

    auto [cost, match] = matching(costs, n, n);
    std::cout << cost << '\n';
}

int main() {
    int32_t n;
    std::cin >> n;

    if (n == 1) {
        std::cout << -1 << '\n';
        return 0;
    }

    std::vector<Sentry> sentries;
    sentries.reserve(n);

    for (auto i = 0; i < n; ++i) {
        P3 p, v;
        std::cin >> p >> v;

        sentries.push_back({p, v});
    }

    if (n % 2 == 1 && allOnSameLine(sentries)) {
        solveLine(std::move(sentries));
    } else {
        solveGeneral(std::move(sentries));
    }

    return 0;
}
